==> netbeans_8.0.2%2Bdfsg1.orig.tar.xz: netbeans-8.0.2/cnd.makeproject/src/org/netbeans/modules/cnd/makeproject/MakeProjectClassPathProvider.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2012 Sun Microsystems, Inc.
 */
package org.netbeans.modules.cnd.makeproject;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.java.classpath.ClassPath;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.modules.cnd.utils.MIMENames;
import org.netbeans.spi.java.classpath.ClassPathProvider;
import org.openide.filesystems.FileObject;
import org.openide.util.WeakSet;

/**
 * Provides ClassPath for CND projects related files. See Bug #215926 for more
 * details.
 *
 * The idea and code is borrowed from
 * org.modules.php.project.classpath.IncludePathClassPathProvider
 *
 * @author inikiforov
 */
//
@org.openide.util.lookup.ServiceProvider(service = ClassPathProvider.class, position = 200)
public class MakeProjectClassPathProvider implements ClassPathProvider {
    private static final Logger LOG = Logger.getLogger(MakeProjectClassPathProvider.class.getName());

    private static final Set<ClassPath> PROJECT_CPS = new WeakSet<>();
    private static final ReadWriteLock PROJECT_LOCK = new ReentrantReadWriteLock();

    public static void addProjectCP(final ClassPath[] cp) {
        runUnderWriteLock(new Runnable() {
            @Override
            public void run() {
                Collections.addAll(PROJECT_CPS, cp);
            }
        });
    }

    public static void removeProjectCP(final ClassPath[] cp) {
        runUnderWriteLock(new Runnable() {
            @Override
            public void run() {
                for (ClassPath classPath : cp) {
                    PROJECT_CPS.remove(classPath);
                }
            }
        });
    }

    @Override
    public ClassPath findClassPath(FileObject file, String type) {
        if (MakeProjectPaths.SOURCES.equals(type)) {
            PROJECT_LOCK.readLock().lock();
            try {
                for (ClassPath spc : PROJECT_CPS) {
                    boolean accept = false;
                    if (spc.contains(file)) {
                        Project owner = FileOwnerQuery.getOwner(file);
                        if (owner instanceof MakeProject) {
                            accept = true;
                        }
                    }
                    if (accept) {
                        LOG.log(Level.FINE, "findClassPath({0}, {1}) -> {2} from {3}", new Object[] {file, type, spc, MakeProjectClassPathProvider.class});
                        return spc;
                    }
                }
            } finally {
                PROJECT_LOCK.readLock().unlock();
            }
        }
        LOG.log(Level.FINE, "findClassPath({0}, {1}) -> null", new Object[] {file, type});
        return null;
    }

    private static void runUnderWriteLock(Runnable runnable) {
        PROJECT_LOCK.writeLock().lock();
        try {
            runnable.run();
        } finally {
            PROJECT_LOCK.writeLock().unlock();
        }
    }
}

==> netbeans_8.2%2Bdfsg1.orig.tar.xz: netbeans-8.2/cnd.makeproject/src/org/netbeans/modules/cnd/makeproject/MakeProjectClassPathProvider.java <==
/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2012 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2012 Sun Microsystems, Inc.
 */
package org.netbeans.modules.cnd.makeproject;

import java.util.Collections;
import java.util.Set;
import java.util.concurrent.locks.ReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.netbeans.api.java.classpath.ClassPath;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.modules.cnd.makeproject.api.MakeProject;
import org.netbeans.spi.java.classpath.ClassPathProvider;
import org.openide.filesystems.FileObject;
import org.openide.util.WeakSet;

/**
 * Provides ClassPath for CND projects related files. See Bug #215926 for more
 * details.
 *
 * The idea and code is borrowed from
 * org.modules.php.project.classpath.IncludePathClassPathProvider
 *
 * @author inikiforov
 */
//
@org.openide.util.lookup.ServiceProvider(service = ClassPathProvider.class, position = 200)
public class MakeProjectClassPathProvider implements ClassPathProvider {
    private static final Logger LOG = Logger.getLogger(MakeProjectClassPathProvider.class.getName());

    private static final Set<ClassPath> PROJECT_CPS = new WeakSet<>();
    private static final ReadWriteLock PROJECT_LOCK = new ReentrantReadWriteLock();

    public static void addProjectCP(final ClassPath[] cp) {
        runUnderWriteLock(() -> {
            Collections.addAll(PROJECT_CPS, cp);
        });
    }

    public static void removeProjectCP(final ClassPath[] cp) {
        runUnderWriteLock(() -> {
            for (ClassPath classPath : cp) {
                PROJECT_CPS.remove(classPath);
            }
        });
    }

    @Override
    public ClassPath findClassPath(FileObject file, String type) {
        if (MakeProjectPaths.SOURCES.equals(type)) {
            PROJECT_LOCK.readLock().lock();
            try {
                for (ClassPath spc : PROJECT_CPS) {
                    boolean accept = false;
                    if (spc.contains(file)) {
                        Project owner = FileOwnerQuery.getOwner(file);
                        if (owner instanceof MakeProject) {
                            accept = true;
                        }
                    }
                    if (accept) {
                        LOG.log(Level.FINE, "findClassPath({0}, {1}) -> {2} from {3}", new Object[] {file, type, spc, MakeProjectClassPathProvider.class});
                        return spc;
                    }
                }
            } finally {
                PROJECT_LOCK.readLock().unlock();
            }
        }
        LOG.log(Level.FINE, "findClassPath({0}, {1}) -> null", new Object[] {file, type});
        return null;
    }

    private static void runUnderWriteLock(Runnable runnable) {
        PROJECT_LOCK.writeLock().lock();
        try {
            runnable.run();
        } finally {
            PROJECT_LOCK.writeLock().unlock();
        }
    }
}
